/*
**  ClanLib SDK
**  Copyright (c) 1997-2010 The ClanLib Team
**
**  This software is provided 'as-is', without any express or implied
**  warranty.  In no event will the authors be held liable for any damages
**  arising from the use of this software.
**
**  Permission is granted to anyone to use this software for any purpose,
**  including commercial applications, and to alter it and redistribute it
**  freely, subject to the following restrictions:
**
**  1. The origin of this software must not be misrepresented; you must not
**     claim that you wrote the original software. If you use this software
**     in a product, an acknowledgment in the product documentation would be
**     appreciated but is not required.
**  2. Altered source versions must be plainly marked as such, and must not be
**     misrepresented as being the original software.
**  3. This notice may not be removed or altered from any source distribution.
**
**  Note: Some of the libraries ClanLib may link to may have additional
**  requirements or restrictions.
**
**  File Author(s):
**
**    Magnus Norddahl
*/

#include "GDI/precomp.h"
#include "API/GDI/setup_gdi.h"
#include "API/GDI/gdi_target.h"

/////////////////////////////////////////////////////////////////////////////
// CL_SetupGDI Construction:

static CL_Mutex cl_gdi_mutex;

static int cl_gdi_refcount = 0;

static CL_GDITarget *cl_gdi_target = 0;

CL_SetupGDI::CL_SetupGDI()
{
	CL_MutexSection mutex_lock(&cl_gdi_mutex);
	if (cl_gdi_refcount == 0)
		cl_gdi_target = new CL_GDITarget();
	cl_gdi_refcount++;
}

CL_SetupGDI::~CL_SetupGDI()
{
	CL_MutexSection mutex_lock(&cl_gdi_mutex);
	cl_gdi_refcount--;
	if (cl_gdi_refcount == 0)
		delete cl_gdi_target;
}

void CL_SetupGDI::set_current()
{
	CL_MutexSection mutex_lock(&cl_gdi_mutex);
	cl_gdi_target->set_current();
}

